@@ -2,22 +2,9 @@
 /*
  * Copyright 2012, 2014 Free Software Foundation, Inc.
  *
- * This file is part of GNU Radio
+ * This file is part of VOLK
  *
- * GNU Radio is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * GNU Radio is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with GNU Radio; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 /*!
@@ -291,21 +278,6 @@ static inline void volk_16ic_magnitude_16i_generic(int16_t* magnitudeVector,
 }
 #endif /* LV_HAVE_GENERIC */
 
-#ifdef LV_HAVE_ORC_DISABLED
-extern void volk_16ic_magnitude_16i_a_orc_impl(int16_t* magnitudeVector,
-                                               const lv_16sc_t* complexVector,
-                                               float scalar,
-                                               unsigned int num_points);
-
-static inline void volk_16ic_magnitude_16i_u_orc(int16_t* magnitudeVector,
-                                                 const lv_16sc_t* complexVector,
-                                                 unsigned int num_points)
-{
-    volk_16ic_magnitude_16i_a_orc_impl(
-        magnitudeVector, complexVector, SHRT_MAX, num_points);
-}
-#endif /* LV_HAVE_ORC */
-
 
 #endif /* INCLUDED_volk_16ic_magnitude_16i_a_H */
 
@@ -439,4 +411,104 @@ static inline void volk_16ic_magnitude_16i_neonv7(int16_t* magnitudeVector,
 }
 #endif /* LV_HAVE_NEONV7 */
 
+#ifdef LV_HAVE_NEONV8
+#include <arm_neon.h>
+
+static inline void volk_16ic_magnitude_16i_neonv8(int16_t* magnitudeVector,
+                                                  const lv_16sc_t* complexVector,
+                                                  unsigned int num_points)
+{
+    unsigned int number = 0;
+    unsigned int quarter_points = num_points / 4;
+
+    const float scalar = SHRT_MAX;
+    const float inv_scalar = 1.0f / scalar;
+
+    int16_t* magnitudeVectorPtr = magnitudeVector;
+    const lv_16sc_t* complexVectorPtr = complexVector;
+
+    float32x4_t mag_vec, mag_sq;
+    float32x4x2_t c_vec;
+
+    for (number = 0; number < quarter_points; number++) {
+        const int16x4x2_t c16_vec = vld2_s16((int16_t*)complexVectorPtr);
+        __VOLK_PREFETCH(complexVectorPtr + 4);
+        c_vec.val[0] = vcvtq_f32_s32(vmovl_s16(c16_vec.val[0]));
+        c_vec.val[1] = vcvtq_f32_s32(vmovl_s16(c16_vec.val[1]));
+        // Scale to close to 0-1
+        c_vec.val[0] = vmulq_n_f32(c_vec.val[0], inv_scalar);
+        c_vec.val[1] = vmulq_n_f32(c_vec.val[1], inv_scalar);
+        // ARMv8: Use FMA for magnitude squared and native sqrt
+        mag_sq =
+            vfmaq_f32(vmulq_f32(c_vec.val[0], c_vec.val[0]), c_vec.val[1], c_vec.val[1]);
+        mag_vec = vsqrtq_f32(mag_sq);
+        // Reconstruct
+        mag_vec = vmulq_n_f32(mag_vec, scalar);
+        // Add 0.5 for correct rounding because vcvtq_s32_f32 truncates.
+        mag_vec = vaddq_f32(mag_vec, vdupq_n_f32(0.5f));
+        const int16x4_t mag16_vec = vmovn_s32(vcvtq_s32_f32(mag_vec));
+        vst1_s16(magnitudeVectorPtr, mag16_vec);
+        // Advance pointers
+        magnitudeVectorPtr += 4;
+        complexVectorPtr += 4;
+    }
+
+    // Deal with the rest
+    for (number = quarter_points * 4; number < num_points; number++) {
+        const float real = lv_creal(*complexVectorPtr) * inv_scalar;
+        const float imag = lv_cimag(*complexVectorPtr) * inv_scalar;
+        *magnitudeVectorPtr =
+            (int16_t)rintf(sqrtf((real * real) + (imag * imag)) * scalar);
+        complexVectorPtr++;
+        magnitudeVectorPtr++;
+    }
+}
+#endif /* LV_HAVE_NEONV8 */
+
+#ifdef LV_HAVE_RVV
+#include <riscv_vector.h>
+
+static inline void volk_16ic_magnitude_16i_rvv(int16_t* magnitudeVector,
+                                               const lv_16sc_t* complexVector,
+                                               unsigned int num_points)
+{
+    const float scale = SHRT_MAX, iscale = 1.0f / scale;
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, complexVector += vl, magnitudeVector += vl) {
+        vl = __riscv_vsetvl_e16m4(n);
+        vint32m8_t vc = __riscv_vle32_v_i32m8((const int32_t*)complexVector, vl);
+        vint16m4_t vr = __riscv_vnsra(vc, 0, vl);
+        vint16m4_t vi = __riscv_vnsra(vc, 16, vl);
+        vfloat32m8_t vrf = __riscv_vfmul(__riscv_vfwcvt_f(vr, vl), iscale, vl);
+        vfloat32m8_t vif = __riscv_vfmul(__riscv_vfwcvt_f(vi, vl), iscale, vl);
+        vfloat32m8_t vf = __riscv_vfmacc(__riscv_vfmul(vif, vif, vl), vrf, vrf, vl);
+        vf = __riscv_vfmul(__riscv_vfsqrt(vf, vl), scale, vl);
+        __riscv_vse16(magnitudeVector, __riscv_vfncvt_x(vf, vl), vl);
+    }
+}
+#endif /*LV_HAVE_RVV*/
+
+#ifdef LV_HAVE_RVVSEG
+#include <riscv_vector.h>
+
+static inline void volk_16ic_magnitude_16i_rvvseg(int16_t* magnitudeVector,
+                                                  const lv_16sc_t* complexVector,
+                                                  unsigned int num_points)
+{
+    const float scale = SHRT_MAX, iscale = 1.0f / scale;
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, complexVector += vl, magnitudeVector += vl) {
+        vl = __riscv_vsetvl_e16m4(n);
+        vint16m4x2_t vc = __riscv_vlseg2e16_v_i16m4x2((const int16_t*)complexVector, vl);
+        vint16m4_t vr = __riscv_vget_i16m4(vc, 0);
+        vint16m4_t vi = __riscv_vget_i16m4(vc, 1);
+        vfloat32m8_t vrf = __riscv_vfmul(__riscv_vfwcvt_f(vr, vl), iscale, vl);
+        vfloat32m8_t vif = __riscv_vfmul(__riscv_vfwcvt_f(vi, vl), iscale, vl);
+        vfloat32m8_t vf = __riscv_vfmacc(__riscv_vfmul(vif, vif, vl), vrf, vrf, vl);
+        vf = __riscv_vfmul(__riscv_vfsqrt(vf, vl), scale, vl);
+        __riscv_vse16(magnitudeVector, __riscv_vfncvt_x(vf, vl), vl);
+    }
+}
+#endif /*LV_HAVE_RVVSEG*/
+
 #endif /* INCLUDED_volk_16ic_magnitude_16i_u_H */
