@@ -2,22 +2,9 @@
 /*
  * Copyright 2016 Free Software Foundation, Inc.
  *
- * This file is part of GNU Radio
+ * This file is part of VOLK
  *
- * GNU Radio is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3, or (at your option)
- * any later version.
- *
- * GNU Radio is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with GNU Radio; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 /*!
@@ -703,4 +690,128 @@ static inline void volk_16ic_x2_dot_prod_16ic_neon_optvma(lv_16sc_t* out,
 
 #endif /* LV_HAVE_NEON */
 
+
+#ifdef LV_HAVE_NEONV8
+#include <arm_neon.h>
+
+static inline void volk_16ic_x2_dot_prod_16ic_neonv8(lv_16sc_t* out,
+                                                     const lv_16sc_t* in_a,
+                                                     const lv_16sc_t* in_b,
+                                                     unsigned int num_points)
+{
+    unsigned int eighth_points = num_points / 8;
+    unsigned int number;
+
+    const lv_16sc_t* a_ptr = in_a;
+    const lv_16sc_t* b_ptr = in_b;
+
+    /* Use 128-bit registers with deinterleaved loads for better throughput */
+    int16x8x2_t a_val, b_val;
+    int16x8_t acc_real1 = vdupq_n_s16(0);
+    int16x8_t acc_real2 = vdupq_n_s16(0);
+    int16x8_t acc_imag1 = vdupq_n_s16(0);
+    int16x8_t acc_imag2 = vdupq_n_s16(0);
+
+    for (number = 0; number < eighth_points; ++number) {
+        a_val = vld2q_s16((int16_t*)a_ptr);
+        b_val = vld2q_s16((int16_t*)b_ptr);
+        __VOLK_PREFETCH(a_ptr + 16);
+        __VOLK_PREFETCH(b_ptr + 16);
+
+        /* real = ar*br - ai*bi, use two accumulators to avoid dependency */
+        acc_real1 = vmlaq_s16(acc_real1, a_val.val[0], b_val.val[0]);
+        acc_real2 = vmlsq_s16(acc_real2, a_val.val[1], b_val.val[1]);
+
+        /* imag = ar*bi + ai*br, use two accumulators */
+        acc_imag1 = vmlaq_s16(acc_imag1, a_val.val[0], b_val.val[1]);
+        acc_imag2 = vmlaq_s16(acc_imag2, a_val.val[1], b_val.val[0]);
+
+        a_ptr += 8;
+        b_ptr += 8;
+    }
+
+    /* Combine accumulators with saturation */
+    int16x8_t acc_real = vqaddq_s16(acc_real1, acc_real2);
+    int16x8_t acc_imag = vqaddq_s16(acc_imag1, acc_imag2);
+
+    /* Horizontal sum using ARMv8 vaddvq */
+    int16_t sum_real = vaddvq_s16(acc_real);
+    int16_t sum_imag = vaddvq_s16(acc_imag);
+
+    *out = lv_cmake(sum_real, sum_imag);
+
+    /* Tail case */
+    for (number = eighth_points * 8; number < num_points; ++number) {
+        lv_16sc_t tmp = in_a[number] * in_b[number];
+        *out = lv_cmake(sat_adds16i(lv_creal(*out), lv_creal(tmp)),
+                        sat_adds16i(lv_cimag(*out), lv_cimag(tmp)));
+    }
+}
+
+#endif /* LV_HAVE_NEONV8 */
+
+
+#ifdef LV_HAVE_RVV
+#include "volk_32fc_x2_dot_prod_32fc.h"
+
+static inline void volk_16ic_x2_dot_prod_16ic_rvv(lv_16sc_t* result,
+                                                  const lv_16sc_t* in_a,
+                                                  const lv_16sc_t* in_b,
+                                                  unsigned int num_points)
+{
+    vint16m4_t vsumr = __riscv_vmv_v_x_i16m4(0, __riscv_vsetvlmax_e16m4());
+    vint16m4_t vsumi = vsumr;
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, in_a += vl, in_b += vl) {
+        vl = __riscv_vsetvl_e16m4(n);
+        vint32m8_t va = __riscv_vle32_v_i32m8((const int32_t*)in_a, vl);
+        vint32m8_t vb = __riscv_vle32_v_i32m8((const int32_t*)in_b, vl);
+        vint16m4_t var = __riscv_vnsra(va, 0, vl), vai = __riscv_vnsra(va, 16, vl);
+        vint16m4_t vbr = __riscv_vnsra(vb, 0, vl), vbi = __riscv_vnsra(vb, 16, vl);
+        vint16m4_t vr = __riscv_vnmsac(__riscv_vmul(var, vbr, vl), vai, vbi, vl);
+        vint16m4_t vi = __riscv_vmacc(__riscv_vmul(var, vbi, vl), vai, vbr, vl);
+        vsumr = __riscv_vadd_tu(vsumr, vsumr, vr, vl);
+        vsumi = __riscv_vadd_tu(vsumi, vsumi, vi, vl);
+    }
+    size_t vl = __riscv_vsetvlmax_e16m1();
+    vint16m1_t vr = RISCV_SHRINK4(vadd, i, 16, vsumr);
+    vint16m1_t vi = RISCV_SHRINK4(vadd, i, 16, vsumi);
+    vint16m1_t z = __riscv_vmv_s_x_i16m1(0, vl);
+    *result = lv_cmake(__riscv_vmv_x(__riscv_vredsum(vr, z, vl)),
+                       __riscv_vmv_x(__riscv_vredsum(vi, z, vl)));
+}
+#endif /*LV_HAVE_RVV*/
+
+#ifdef LV_HAVE_RVVSEG
+#include "volk_32fc_x2_dot_prod_32fc.h"
+
+
+static inline void volk_16ic_x2_dot_prod_16ic_rvvseg(lv_16sc_t* result,
+                                                     const lv_16sc_t* in_a,
+                                                     const lv_16sc_t* in_b,
+                                                     unsigned int num_points)
+{
+    vint16m4_t vsumr = __riscv_vmv_v_x_i16m4(0, __riscv_vsetvlmax_e16m4());
+    vint16m4_t vsumi = vsumr;
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, in_a += vl, in_b += vl) {
+        vl = __riscv_vsetvl_e16m4(n);
+        vint16m4x2_t va = __riscv_vlseg2e16_v_i16m4x2((const int16_t*)in_a, vl);
+        vint16m4x2_t vb = __riscv_vlseg2e16_v_i16m4x2((const int16_t*)in_b, vl);
+        vint16m4_t var = __riscv_vget_i16m4(va, 0), vai = __riscv_vget_i16m4(va, 1);
+        vint16m4_t vbr = __riscv_vget_i16m4(vb, 0), vbi = __riscv_vget_i16m4(vb, 1);
+        vint16m4_t vr = __riscv_vnmsac(__riscv_vmul(var, vbr, vl), vai, vbi, vl);
+        vint16m4_t vi = __riscv_vmacc(__riscv_vmul(var, vbi, vl), vai, vbr, vl);
+        vsumr = __riscv_vadd_tu(vsumr, vsumr, vr, vl);
+        vsumi = __riscv_vadd_tu(vsumi, vsumi, vi, vl);
+    }
+    size_t vl = __riscv_vsetvlmax_e16m1();
+    vint16m1_t vr = RISCV_SHRINK4(vadd, i, 16, vsumr);
+    vint16m1_t vi = RISCV_SHRINK4(vadd, i, 16, vsumi);
+    vint16m1_t z = __riscv_vmv_s_x_i16m1(0, vl);
+    *result = lv_cmake(__riscv_vmv_x(__riscv_vredsum(vr, z, vl)),
+                       __riscv_vmv_x(__riscv_vredsum(vi, z, vl)));
+}
+#endif /*LV_HAVE_RVVSEG*/
+
 #endif /*INCLUDED_volk_16ic_x2_dot_prod_16ic_H*/
