@@ -171,6 +171,38 @@ static inline void volk_32f_s32f_add_32f_u_neon(float* cVector,
 }
 #endif /* LV_HAVE_NEON */
 
+#ifdef LV_HAVE_NEONV8
+#include <arm_neon.h>
+
+static inline void volk_32f_s32f_add_32f_neonv8(float* cVector,
+                                                const float* aVector,
+                                                const float scalar,
+                                                unsigned int num_points)
+{
+    const unsigned int eighthPoints = num_points / 8;
+
+    const float* aPtr = aVector;
+    float* cPtr = cVector;
+    const float32x4_t scalarVec = vdupq_n_f32(scalar);
+
+    for (unsigned int number = 0; number < eighthPoints; number++) {
+        float32x4_t a0 = vld1q_f32(aPtr);
+        float32x4_t a1 = vld1q_f32(aPtr + 4);
+        __VOLK_PREFETCH(aPtr + 16);
+
+        vst1q_f32(cPtr, vaddq_f32(a0, scalarVec));
+        vst1q_f32(cPtr + 4, vaddq_f32(a1, scalarVec));
+
+        aPtr += 8;
+        cPtr += 8;
+    }
+
+    for (unsigned int number = eighthPoints * 8; number < num_points; number++) {
+        *cPtr++ = (*aPtr++) + scalar;
+    }
+}
+#endif /* LV_HAVE_NEONV8 */
+
 
 #endif /* INCLUDED_volk_32f_s32f_add_32f_u_H */
 
@@ -247,7 +279,7 @@ static inline void volk_32f_s32f_add_32f_a_avx(float* cVector,
 extern void volk_32f_s32f_add_32f_a_orc_impl(float* dst,
                                              const float* src,
                                              const float scalar,
-                                             unsigned int num_points);
+                                             int num_points);
 
 static inline void volk_32f_s32f_add_32f_u_orc(float* cVector,
                                                const float* aVector,
@@ -258,4 +290,21 @@ static inline void volk_32f_s32f_add_32f_u_orc(float* cVector,
 }
 #endif /* LV_HAVE_ORC */
 
+#ifdef LV_HAVE_RVV
+#include <riscv_vector.h>
+
+static inline void volk_32f_s32f_add_32f_rvv(float* cVector,
+                                             const float* aVector,
+                                             const float scalar,
+                                             unsigned int num_points)
+{
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, aVector += vl, cVector += vl) {
+        vl = __riscv_vsetvl_e32m8(n);
+        vfloat32m8_t v = __riscv_vle32_v_f32m8(aVector, vl);
+        __riscv_vse32(cVector, __riscv_vfadd(v, scalar, vl), vl);
+    }
+}
+#endif /*LV_HAVE_RVV*/
+
 #endif /* INCLUDED_volk_32f_s32f_add_32f_a_H */
