@@ -62,6 +62,73 @@
 #include <math.h>
 #include <stdio.h>
 
+#ifdef LV_HAVE_GENERIC
+
+static inline void volk_32fc_magnitude_32f_generic(float* magnitudeVector,
+                                                   const lv_32fc_t* complexVector,
+                                                   unsigned int num_points)
+{
+    const float* complexVectorPtr = (float*)complexVector;
+    float* magnitudeVectorPtr = magnitudeVector;
+    unsigned int number = 0;
+    for (number = 0; number < num_points; number++) {
+        const float real = *complexVectorPtr++;
+        const float imag = *complexVectorPtr++;
+        *magnitudeVectorPtr++ = sqrtf((real * real) + (imag * imag));
+    }
+}
+#endif /* LV_HAVE_GENERIC */
+
+#ifdef LV_HAVE_AVX512F
+#include <immintrin.h>
+
+static inline void volk_32fc_magnitude_32f_u_avx512f(float* magnitudeVector,
+                                                     const lv_32fc_t* complexVector,
+                                                     unsigned int num_points)
+{
+    unsigned int number = 0;
+    const unsigned int eighthPoints = num_points / 8;
+
+    const float* complexVectorPtr = (float*)complexVector;
+    float* magnitudeVectorPtr = magnitudeVector;
+
+    __m512 cplxValue;
+    __m512 iValues, qValues;
+    __m512 result;
+
+    for (; number < eighthPoints; number++) {
+        // Load 8 complex numbers (16 floats): I0,Q0,I1,Q1,...,I7,Q7
+        cplxValue = _mm512_loadu_ps(complexVectorPtr);
+
+        // Separate I and Q values using permute
+        // Extract I values (even indices: 0,2,4,6,8,10,12,14)
+        iValues = _mm512_permutexvar_ps(
+            _mm512_setr_epi32(0, 2, 4, 6, 8, 10, 12, 14, 0, 0, 0, 0, 0, 0, 0, 0),
+            cplxValue);
+        // Extract Q values (odd indices: 1,3,5,7,9,11,13,15)
+        qValues = _mm512_permutexvar_ps(
+            _mm512_setr_epi32(1, 3, 5, 7, 9, 11, 13, 15, 0, 0, 0, 0, 0, 0, 0, 0),
+            cplxValue);
+
+        // Square and add: I^2 + Q^2
+        result = _mm512_fmadd_ps(iValues, iValues, _mm512_mul_ps(qValues, qValues));
+
+        // Square root
+        result = _mm512_sqrt_ps(result);
+
+        // Store only first 8 results
+        _mm256_storeu_ps(magnitudeVectorPtr, _mm512_castps512_ps256(result));
+
+        complexVectorPtr += 16;
+        magnitudeVectorPtr += 8;
+    }
+
+    number = eighthPoints * 8;
+    volk_32fc_magnitude_32f_generic(
+        magnitudeVectorPtr, (const lv_32fc_t*)complexVectorPtr, num_points - number);
+}
+#endif /* LV_HAVE_AVX512F */
+
 #ifdef LV_HAVE_AVX
 #include <immintrin.h>
 #include <volk/volk_avx_intrinsics.h>
@@ -172,25 +239,6 @@ static inline void volk_32fc_magnitude_32f_u_sse(float* magnitudeVector,
 }
 #endif /* LV_HAVE_SSE */
 
-
-#ifdef LV_HAVE_GENERIC
-
-static inline void volk_32fc_magnitude_32f_generic(float* magnitudeVector,
-                                                   const lv_32fc_t* complexVector,
-                                                   unsigned int num_points)
-{
-    const float* complexVectorPtr = (float*)complexVector;
-    float* magnitudeVectorPtr = magnitudeVector;
-    unsigned int number = 0;
-    for (number = 0; number < num_points; number++) {
-        const float real = *complexVectorPtr++;
-        const float imag = *complexVectorPtr++;
-        *magnitudeVectorPtr++ = sqrtf((real * real) + (imag * imag));
-    }
-}
-#endif /* LV_HAVE_GENERIC */
-
-
 #endif /* INCLUDED_volk_32fc_magnitude_32f_u_H */
 #ifndef INCLUDED_volk_32fc_magnitude_32f_a_H
 #define INCLUDED_volk_32fc_magnitude_32f_a_H
@@ -199,6 +247,54 @@ static inline void volk_32fc_magnitude_32f_generic(float* magnitudeVector,
 #include <math.h>
 #include <stdio.h>
 
+#ifdef LV_HAVE_AVX512F
+#include <immintrin.h>
+
+static inline void volk_32fc_magnitude_32f_a_avx512f(float* magnitudeVector,
+                                                     const lv_32fc_t* complexVector,
+                                                     unsigned int num_points)
+{
+    unsigned int number = 0;
+    const unsigned int eighthPoints = num_points / 8;
+
+    const float* complexVectorPtr = (float*)complexVector;
+    float* magnitudeVectorPtr = magnitudeVector;
+
+    __m512 cplxValue;
+    __m512 iValues, qValues;
+    __m512 result;
+
+    for (; number < eighthPoints; number++) {
+        // Load 8 complex numbers (16 floats): I0,Q0,I1,Q1,...,I7,Q7
+        cplxValue = _mm512_load_ps(complexVectorPtr);
+
+        // Separate I and Q values using permute
+        iValues = _mm512_permutexvar_ps(
+            _mm512_setr_epi32(0, 2, 4, 6, 8, 10, 12, 14, 0, 0, 0, 0, 0, 0, 0, 0),
+            cplxValue);
+        qValues = _mm512_permutexvar_ps(
+            _mm512_setr_epi32(1, 3, 5, 7, 9, 11, 13, 15, 0, 0, 0, 0, 0, 0, 0, 0),
+            cplxValue);
+
+        // Square and add: I^2 + Q^2
+        result = _mm512_fmadd_ps(iValues, iValues, _mm512_mul_ps(qValues, qValues));
+
+        // Square root
+        result = _mm512_sqrt_ps(result);
+
+        // Store only first 8 results
+        _mm256_store_ps(magnitudeVectorPtr, _mm512_castps512_ps256(result));
+
+        complexVectorPtr += 16;
+        magnitudeVectorPtr += 8;
+    }
+
+    number = eighthPoints * 8;
+    volk_32fc_magnitude_32f_generic(
+        magnitudeVectorPtr, (const lv_32fc_t*)complexVectorPtr, num_points - number);
+}
+#endif /* LV_HAVE_AVX512F */
+
 #ifdef LV_HAVE_AVX
 #include <immintrin.h>
 #include <volk/volk_avx_intrinsics.h>
@@ -308,24 +404,6 @@ static inline void volk_32fc_magnitude_32f_a_sse(float* magnitudeVector,
 #endif /* LV_HAVE_SSE */
 
 
-#ifdef LV_HAVE_GENERIC
-
-static inline void volk_32fc_magnitude_32f_a_generic(float* magnitudeVector,
-                                                     const lv_32fc_t* complexVector,
-                                                     unsigned int num_points)
-{
-    const float* complexVectorPtr = (float*)complexVector;
-    float* magnitudeVectorPtr = magnitudeVector;
-    unsigned int number = 0;
-    for (number = 0; number < num_points; number++) {
-        const float real = *complexVectorPtr++;
-        const float imag = *complexVectorPtr++;
-        *magnitudeVectorPtr++ = sqrtf((real * real) + (imag * imag));
-    }
-}
-#endif /* LV_HAVE_GENERIC */
-
-
 #ifdef LV_HAVE_NEON
 #include <arm_neon.h>
 
@@ -362,6 +440,44 @@ static inline void volk_32fc_magnitude_32f_neon(float* magnitudeVector,
 #endif /* LV_HAVE_NEON */
 
 
+#ifdef LV_HAVE_NEONV8
+#include <arm_neon.h>
+
+static inline void volk_32fc_magnitude_32f_neonv8(float* magnitudeVector,
+                                                  const lv_32fc_t* complexVector,
+                                                  unsigned int num_points)
+{
+    unsigned int number;
+    unsigned int quarter_points = num_points / 4;
+    const float* complexVectorPtr = (float*)complexVector;
+    float* magnitudeVectorPtr = magnitudeVector;
+
+    float32x4x2_t complex_vec;
+    float32x4_t magnitude_sq, magnitude_vec;
+    for (number = 0; number < quarter_points; number++) {
+        complex_vec = vld2q_f32(complexVectorPtr);
+        __VOLK_PREFETCH(complexVectorPtr + 8);
+        // Use FMA for magnitude squared: real^2 + imag^2
+        magnitude_sq = vfmaq_f32(vmulq_f32(complex_vec.val[0], complex_vec.val[0]),
+                                 complex_vec.val[1],
+                                 complex_vec.val[1]);
+        // ARMv8 native sqrt
+        magnitude_vec = vsqrtq_f32(magnitude_sq);
+        vst1q_f32(magnitudeVectorPtr, magnitude_vec);
+
+        complexVectorPtr += 8;
+        magnitudeVectorPtr += 4;
+    }
+
+    for (number = quarter_points * 4; number < num_points; number++) {
+        const float real = *complexVectorPtr++;
+        const float imag = *complexVectorPtr++;
+        *magnitudeVectorPtr++ = sqrtf((real * real) + (imag * imag));
+    }
+}
+#endif /* LV_HAVE_NEONV8 */
+
+
 #ifdef LV_HAVE_NEON
 /*!
   \brief Calculates the magnitude of the complexVector and stores the results in the
@@ -438,20 +554,42 @@ static inline void volk_32fc_magnitude_32f_neon_fancy_sweet(
 }
 #endif /* LV_HAVE_NEON */
 
+#ifdef LV_HAVE_RVV
+#include <riscv_vector.h>
 
-#ifdef LV_HAVE_ORC
-
-extern void volk_32fc_magnitude_32f_a_orc_impl(float* magnitudeVector,
+static inline void volk_32fc_magnitude_32f_rvv(float* magnitudeVector,
                                                const lv_32fc_t* complexVector,
-                                               unsigned int num_points);
-
-static inline void volk_32fc_magnitude_32f_u_orc(float* magnitudeVector,
-                                                 const lv_32fc_t* complexVector,
-                                                 unsigned int num_points)
+                                               unsigned int num_points)
 {
-    volk_32fc_magnitude_32f_a_orc_impl(magnitudeVector, complexVector, num_points);
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, complexVector += vl, magnitudeVector += vl) {
+        vl = __riscv_vsetvl_e32m4(n);
+        vuint64m8_t vc = __riscv_vle64_v_u64m8((const uint64_t*)complexVector, vl);
+        vfloat32m4_t vr = __riscv_vreinterpret_f32m4(__riscv_vnsrl(vc, 0, vl));
+        vfloat32m4_t vi = __riscv_vreinterpret_f32m4(__riscv_vnsrl(vc, 32, vl));
+        vfloat32m4_t v = __riscv_vfmacc(__riscv_vfmul(vi, vi, vl), vr, vr, vl);
+        __riscv_vse32(magnitudeVector, __riscv_vfsqrt(v, vl), vl);
+    }
 }
-#endif /* LV_HAVE_ORC */
+#endif /*LV_HAVE_RVV*/
 
+#ifdef LV_HAVE_RVVSEG
+#include <riscv_vector.h>
+
+static inline void volk_32fc_magnitude_32f_rvvseg(float* magnitudeVector,
+                                                  const lv_32fc_t* complexVector,
+                                                  unsigned int num_points)
+{
+    size_t n = num_points;
+    for (size_t vl; n > 0; n -= vl, complexVector += vl, magnitudeVector += vl) {
+        vl = __riscv_vsetvl_e32m4(n);
+        vfloat32m4x2_t vc = __riscv_vlseg2e32_v_f32m4x2((const float*)complexVector, vl);
+        vfloat32m4_t vr = __riscv_vget_f32m4(vc, 0);
+        vfloat32m4_t vi = __riscv_vget_f32m4(vc, 1);
+        vfloat32m4_t v = __riscv_vfmacc(__riscv_vfmul(vi, vi, vl), vr, vr, vl);
+        __riscv_vse32(magnitudeVector, __riscv_vfsqrt(v, vl), vl);
+    }
+}
+#endif /*LV_HAVE_RVVSEG*/
 
 #endif /* INCLUDED_volk_32fc_magnitude_32f_a_H */
