@@ -55,45 +55,6 @@ static inline void volk_32fc_s32fc_rotator2puppet_32fc_neon(lv_32fc_t* outVector
 #endif /* LV_HAVE_NEON */
 
 
-#ifdef LV_HAVE_SSE4_1
-#include <smmintrin.h>
-
-static inline void
-volk_32fc_s32fc_rotator2puppet_32fc_a_sse4_1(lv_32fc_t* outVector,
-                                             const lv_32fc_t* inVector,
-                                             const lv_32fc_t* phase_inc,
-                                             unsigned int num_points)
-{
-    lv_32fc_t phase[1] = { lv_cmake(.3f, .95393f) };
-    (*phase) /= hypotf(lv_creal(*phase), lv_cimag(*phase));
-    const lv_32fc_t phase_inc_n =
-        *phase_inc / hypotf(lv_creal(*phase_inc), lv_cimag(*phase_inc));
-    volk_32fc_s32fc_x2_rotator2_32fc_a_sse4_1(
-        outVector, inVector, &phase_inc_n, phase, num_points);
-}
-
-#endif /* LV_HAVE_SSE4_1 */
-
-
-#ifdef LV_HAVE_SSE4_1
-#include <smmintrin.h>
-static inline void
-volk_32fc_s32fc_rotator2puppet_32fc_u_sse4_1(lv_32fc_t* outVector,
-                                             const lv_32fc_t* inVector,
-                                             const lv_32fc_t* phase_inc,
-                                             unsigned int num_points)
-{
-    lv_32fc_t phase[1] = { lv_cmake(.3f, .95393f) };
-    (*phase) /= hypotf(lv_creal(*phase), lv_cimag(*phase));
-    const lv_32fc_t phase_inc_n =
-        *phase_inc / hypotf(lv_creal(*phase_inc), lv_cimag(*phase_inc));
-    volk_32fc_s32fc_x2_rotator2_32fc_u_sse4_1(
-        outVector, inVector, &phase_inc_n, phase, num_points);
-}
-
-#endif /* LV_HAVE_SSE4_1 */
-
-
 #ifdef LV_HAVE_AVX
 #include <immintrin.h>
 
@@ -131,11 +92,12 @@ static inline void volk_32fc_s32fc_rotator2puppet_32fc_u_avx(lv_32fc_t* outVecto
 
 #endif /* LV_HAVE_AVX */
 
-#if LV_HAVE_AVX && LV_HAVE_FMA
+
+#ifdef LV_HAVE_AVX512F
 #include <immintrin.h>
 
 static inline void
-volk_32fc_s32fc_rotator2puppet_32fc_a_avx_fma(lv_32fc_t* outVector,
+volk_32fc_s32fc_rotator2puppet_32fc_a_avx512f(lv_32fc_t* outVector,
                                               const lv_32fc_t* inVector,
                                               const lv_32fc_t* phase_inc,
                                               unsigned int num_points)
@@ -144,18 +106,18 @@ volk_32fc_s32fc_rotator2puppet_32fc_a_avx_fma(lv_32fc_t* outVector,
     (*phase) /= hypotf(lv_creal(*phase), lv_cimag(*phase));
     const lv_32fc_t phase_inc_n =
         *phase_inc / hypotf(lv_creal(*phase_inc), lv_cimag(*phase_inc));
-    volk_32fc_s32fc_x2_rotator2_32fc_a_avx_fma(
+    volk_32fc_s32fc_x2_rotator2_32fc_a_avx512f(
         outVector, inVector, &phase_inc_n, phase, num_points);
 }
 
-#endif /* LV_HAVE_AVX && LV_HAVE_FMA*/
+#endif /* LV_HAVE_AVX512F */
 
 
-#if LV_HAVE_AVX && LV_HAVE_FMA
+#ifdef LV_HAVE_AVX512F
 #include <immintrin.h>
 
 static inline void
-volk_32fc_s32fc_rotator2puppet_32fc_u_avx_fma(lv_32fc_t* outVector,
+volk_32fc_s32fc_rotator2puppet_32fc_u_avx512f(lv_32fc_t* outVector,
                                               const lv_32fc_t* inVector,
                                               const lv_32fc_t* phase_inc,
                                               unsigned int num_points)
@@ -164,11 +126,11 @@ volk_32fc_s32fc_rotator2puppet_32fc_u_avx_fma(lv_32fc_t* outVector,
     (*phase) /= hypotf(lv_creal(*phase), lv_cimag(*phase));
     const lv_32fc_t phase_inc_n =
         *phase_inc / hypotf(lv_creal(*phase_inc), lv_cimag(*phase_inc));
-    volk_32fc_s32fc_x2_rotator2_32fc_u_avx_fma(
+    volk_32fc_s32fc_x2_rotator2_32fc_u_avx512f(
         outVector, inVector, &phase_inc_n, phase, num_points);
 }
 
-#endif /* LV_HAVE_AVX && LV_HAVE_FMA*/
+#endif /* LV_HAVE_AVX512F */
 
 #ifdef LV_HAVE_RVV
 static inline void volk_32fc_s32fc_rotator2puppet_32fc_rvv(lv_32fc_t* outVector,
