@@ -35,6 +35,17 @@ static inline void volk_32u_popcntpuppet_32u_a_sse4_2(uint32_t* outVector,
 }
 #endif /* LV_HAVE_SSE4_2 */
 
+#ifdef LV_HAVE_NEON
+static inline void volk_32u_popcntpuppet_32u_neon(uint32_t* outVector,
+                                                  const uint32_t* inVector,
+                                                  unsigned int num_points)
+{
+    for (size_t i = 0; i < num_points; ++i) {
+        volk_32u_popcnt_neon(outVector + i, inVector[i]);
+    }
+}
+#endif /* LV_HAVE_NEON */
+
 #ifdef LV_HAVE_RVV
 static inline void volk_32u_popcntpuppet_32u_rvv(uint32_t* outVector,
                                                  const uint32_t* inVector,
