@@ -1,7 +1,7 @@
 /* -*- c++ -*- */
 /*
  * Copyright 2014 - 2021 Free Software Foundation, Inc.
- * Copyright 2023, 2024 Magnus Lundmark <magnuslundmark@gmail.com>
+ * Copyright 2023 - 2025 Magnus Lundmark <magnuslundmark@gmail.com>
  *
  * This file is part of VOLK
  *
@@ -11,9 +11,11 @@
 #include "qa_utils.h"
 
 #include <volk/volk.h>
+#include <cmath>
+#include <limits>
 #include <vector>
 
-// macros for initializing volk_test_case_t. Maccros are needed to generate
+// macros for initializing volk_test_case_t. Macros are needed to generate
 // function names of the pattern kernel_name_*
 
 // for puppets we need to get all the func_variants for the puppet and just
@@ -34,6 +36,8 @@
 #define QA(test) test_cases.push_back(test);
 std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
 {
+    const float inf = std::numeric_limits<float>::infinity();
+    const float nan = std::nanf("");
 
     // Some kernels need a lower tolerance
     volk_test_params_t test_params_inacc = test_params.make_tol(1e-2);
@@ -82,23 +86,97 @@ std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
     QA(VOLK_INIT_TEST(volk_16i_32fc_dot_prod_32fc, test_params.make_absolute(1e-1)))
     QA(VOLK_INIT_TEST(volk_32f_accumulator_s32f, test_params.make_absolute(2e-2)))
     QA(VOLK_INIT_TEST(volk_32f_x2_add_32f, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_index_max_16u, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_index_max_32u, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_index_min_16u, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_index_min_32u, test_params))
+
+    // Index kernels need identical values to test tie-breaking (first index wins)
+    volk_test_params_t test_params_index(test_params.make_tol(0));
+    test_params_index.add_float_edge_cases({
+        1.0f,
+        1.0f,
+        1.0f,
+        1.0f, // 4 identical (SSE lane width)
+        1.0f,
+        1.0f,
+        1.0f,
+        1.0f, // 8 total (AVX lane width)
+        1.0f,
+        1.0f,
+        1.0f,
+        1.0f, // 12
+        1.0f,
+        1.0f,
+        1.0f,
+        1.0f, // 16 total (AVX512 lane width)
+    });
+    QA(VOLK_INIT_TEST(volk_32f_index_max_16u, test_params_index))
+    QA(VOLK_INIT_TEST(volk_32f_index_max_32u, test_params_index))
+    QA(VOLK_INIT_TEST(volk_32f_index_min_16u, test_params_index))
+    QA(VOLK_INIT_TEST(volk_32f_index_min_32u, test_params_index))
     QA(VOLK_INIT_TEST(volk_32fc_32f_multiply_32fc, test_params))
     QA(VOLK_INIT_TEST(volk_32fc_32f_add_32fc, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_log2_32f, test_params.make_absolute(1.5e-5)))
+
+    volk_test_params_t test_params_log2(test_params.make_absolute(5e-6));
+    test_params_log2.add_float_edge_cases({ -1.f, 0.f, inf, 65536.f });
+    QA(VOLK_INIT_TEST(volk_32f_log2_32f, test_params_log2))
+
     QA(VOLK_INIT_TEST(volk_32f_expfast_32f, test_params_inacc_tenth))
-    QA(VOLK_INIT_TEST(volk_32f_sin_32f, test_params_inacc))
-    QA(VOLK_INIT_TEST(volk_32f_cos_32f, test_params_inacc))
+    QA(VOLK_INIT_TEST(volk_32f_sin_32f, test_params))
+    QA(VOLK_INIT_TEST(volk_32f_cos_32f, test_params))
+    QA(VOLK_INIT_TEST(volk_32f_sincos_32f_x2, test_params))
     QA(VOLK_INIT_TEST(volk_32f_tan_32f, test_params_inacc))
-    QA(VOLK_INIT_TEST(volk_32f_atan_32f, test_params))
-    QA(VOLK_INIT_TEST(volk_32f_asin_32f, test_params_inacc))
-    QA(VOLK_INIT_TEST(volk_32f_acos_32f, test_params_inacc))
+
+    volk_test_params_t test_params_atan(test_params);
+    test_params_atan.add_float_edge_cases({ std::nanf(""),
+                                            std::numeric_limits<float>::infinity(),
+                                            -std::numeric_limits<float>::infinity(),
+                                            0.0f,
+                                            -0.0f,
+                                            1e10f,
+                                            -1e10f,
+                                            1.0f,
+                                            -1.0f });
+    QA(VOLK_INIT_TEST(volk_32f_atan_32f, test_params_atan))
+
+    volk_test_params_t test_params_asin(test_params);
+    test_params_asin.set_tol(1e-5);
+    test_params_asin.add_float_edge_cases({ std::nanf(""),
+                                            1.0f,
+                                            -1.0f,
+                                            0.0f,
+                                            -0.0f,
+                                            0.5f,
+                                            -0.5f,
+                                            0.99f,
+                                            -0.99f,
+                                            0.707107f,
+                                            -0.707107f });
+    QA(VOLK_INIT_TEST(volk_32f_asin_32f, test_params_asin))
+    QA(VOLK_INIT_TEST(volk_32f_acos_32f, test_params_asin))
     QA(VOLK_INIT_TEST(volk_32fc_s32f_power_32fc, test_params_power))
     QA(VOLK_INIT_TEST(volk_32f_s32f_calc_spectral_noise_floor_32f, test_params_snf))
-    QA(VOLK_INIT_TEST(volk_32fc_s32f_atan2_32f, test_params))
+
+    volk_test_params_t test_params_atan2(test_params);
+    test_params_atan2.add_complex_edge_cases(
+        { lv_cmake(0.0f, 0.0f),   // atan2(0, 0) = 0
+          lv_cmake(0.0f, -0.0f),  // atan2(-0, 0) = -0 (preserve sign)
+          lv_cmake(0.0f, 1.0f),   // atan2(1, 0) = π/2
+          lv_cmake(0.0f, -1.0f),  // atan2(-1, 0) = -π/2
+          lv_cmake(1.0f, 0.0f),   // atan2(0, 1) = 0
+          lv_cmake(-1.0f, 0.0f),  // atan2(0, -1) = π
+          lv_cmake(1.0f, 1.0f),   // atan2(1, 1) = π/4
+          lv_cmake(-1.0f, 1.0f),  // atan2(1, -1) = 3π/4
+          lv_cmake(-1.0f, -1.0f), // atan2(-1, -1) = -3π/4
+          lv_cmake(1.0f, -1.0f),  // atan2(-1, 1) = -π/4
+          lv_cmake(inf, inf),     // atan2(inf, inf) = π/4
+          lv_cmake(inf, -inf),    // atan2(-inf, inf) = -π/4
+          lv_cmake(-inf, inf),    // atan2(inf, -inf) = 3π/4
+          lv_cmake(-inf, -inf),   // atan2(-inf, -inf) = -3π/4
+          lv_cmake(inf, 0.0f),    // atan2(0, inf) = 0
+          lv_cmake(-inf, 0.0f),   // atan2(0, -inf) = π
+          lv_cmake(1.0f, inf),    // atan2(inf, 1) = π/2
+          lv_cmake(1.0f, -inf),   // atan2(-inf, 1) = -π/2
+          lv_cmake(nan, 1.0f),    // atan2(1, nan) = nan (propagate)
+          lv_cmake(1.0f, nan) }); // atan2(nan, 1) = nan (propagate)
+    QA(VOLK_INIT_TEST(volk_32fc_s32f_atan2_32f, test_params_atan2))
     QA(VOLK_INIT_TEST(volk_32fc_x2_conjugate_dot_prod_32fc,
                       test_params.make_absolute(2e-2)))
     QA(VOLK_INIT_TEST(volk_32fc_deinterleave_32f_x2, test_params))
@@ -110,10 +188,23 @@ std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
     QA(VOLK_INIT_TEST(volk_32fc_deinterleave_real_64f, test_params))
     QA(VOLK_INIT_TEST(volk_32fc_x2_dot_prod_32fc, test_params.make_absolute(2e-2)))
     QA(VOLK_INIT_TEST(volk_32fc_32f_dot_prod_32fc, test_params.make_absolute(1e-2)))
-    QA(VOLK_INIT_TEST(volk_32fc_index_max_16u, test_params))
-    QA(VOLK_INIT_TEST(volk_32fc_index_max_32u, test_params))
-    QA(VOLK_INIT_TEST(volk_32fc_index_min_16u, test_params))
-    QA(VOLK_INIT_TEST(volk_32fc_index_min_32u, test_params))
+
+    // Complex index kernels: same magnitude values to test tie-breaking
+    volk_test_params_t test_params_index_fc(test_params.make_tol(0));
+    test_params_index_fc.add_complex_edge_cases({
+        lv_cmake(1.0f, 0.0f),
+        lv_cmake(1.0f, 0.0f), // 2 same magnitude
+        lv_cmake(0.0f, 1.0f),
+        lv_cmake(0.0f, 1.0f), // 4 (all |z|=1)
+        lv_cmake(1.0f, 0.0f),
+        lv_cmake(1.0f, 0.0f), // 6
+        lv_cmake(0.0f, 1.0f),
+        lv_cmake(0.0f, 1.0f), // 8 (covers AVX 8-wide)
+    });
+    QA(VOLK_INIT_TEST(volk_32fc_index_max_16u, test_params_index_fc))
+    QA(VOLK_INIT_TEST(volk_32fc_index_max_32u, test_params_index_fc))
+    QA(VOLK_INIT_TEST(volk_32fc_index_min_16u, test_params_index_fc))
+    QA(VOLK_INIT_TEST(volk_32fc_index_min_32u, test_params_index_fc))
     QA(VOLK_INIT_TEST(volk_32fc_s32f_magnitude_16i, test_params.make_tol(1)))
     QA(VOLK_INIT_TEST(volk_32fc_magnitude_32f, test_params_inacc_tenth))
     QA(VOLK_INIT_TEST(volk_32fc_magnitude_squared_32f, test_params))
@@ -143,6 +234,11 @@ std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
     QA(VOLK_INIT_TEST(volk_32f_s32f_power_32f, test_params))
     QA(VOLK_INIT_TEST(volk_32f_reciprocal_32f, test_params.make_tol(6.15e-5)))
     QA(VOLK_INIT_TEST(volk_32f_sqrt_32f, test_params_inacc))
+
+    volk_test_params_t test_params_invsqrt(test_params.make_tol(1e-6));
+    test_params_invsqrt.add_float_edge_cases(
+        { -1.f, 1.f, 0.f, inf, 1e-2f, 1e2f, 1e-10, 1e10 });
+    QA(VOLK_INIT_TEST(volk_32f_invsqrt_32f, test_params_invsqrt))
     QA(VOLK_INIT_TEST(volk_32f_s32f_stddev_32f, test_params_inacc))
     QA(VOLK_INIT_TEST(volk_32f_stddev_and_mean_32f_x2, test_params.make_absolute(1e-5)))
     QA(VOLK_INIT_TEST(volk_32f_x2_subtract_32f, test_params))
@@ -156,6 +252,7 @@ std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
     QA(VOLK_INIT_TEST(volk_64f_x2_min_64f, test_params))
     QA(VOLK_INIT_TEST(volk_64f_x2_multiply_64f, test_params))
     QA(VOLK_INIT_TEST(volk_64f_x2_add_64f, test_params))
+    QA(VOLK_INIT_TEST(volk_64f_x2_dot_prod_64f, test_params))
     QA(VOLK_INIT_TEST(volk_8ic_deinterleave_16i_x2, test_params))
     QA(VOLK_INIT_TEST(volk_8ic_s32f_deinterleave_32f_x2, test_params))
     QA(VOLK_INIT_TEST(volk_8ic_deinterleave_real_16i, test_params))
@@ -165,6 +262,10 @@ std::vector<volk_test_case_t> init_test_list(volk_test_params_t test_params)
     QA(VOLK_INIT_TEST(volk_8ic_x2_s32f_multiply_conjugate_32fc, test_params))
     QA(VOLK_INIT_TEST(volk_8i_convert_16i, test_params))
     QA(VOLK_INIT_TEST(volk_8i_s32f_convert_32f, test_params))
+    QA(VOLK_INIT_TEST(volk_8i_x2_add_saturated_8i, test_params))
+    QA(VOLK_INIT_TEST(volk_8u_x2_add_saturated_8u, test_params))
+    QA(VOLK_INIT_TEST(volk_16i_x2_add_saturated_16i, test_params))
+    QA(VOLK_INIT_TEST(volk_16u_x2_add_saturated_16u, test_params))
     QA(VOLK_INIT_TEST(volk_32fc_s32fc_multiply2_32fc, test_params))
     QA(VOLK_INIT_TEST(volk_32f_s32f_multiply_32f, test_params))
     QA(VOLK_INIT_TEST(volk_32f_s32f_add_32f, test_params))
